@@ -62,31 +62,67 @@ int seek_read( const int fd, uint8_t * const buf, const int size,
   return 0;
   }
 
-// Check archive type, remove EOF blocks, and leave outfd file pos at EOF
-bool check_appendable()
+// infd and outfd can refer to the same file if copying to a lower file
+// position or if source and destination blocks don't overlap.
+// max_size < 0 means no size limit.
+bool copy_file( const int infd, const int outfd, const long long max_size = -1 )
+  {
+  const int buffer_size = 65536;
+  // remaining number of bytes to copy
+  long long rest = ( ( max_size >= 0 ) ? max_size : buffer_size );
+  long long copied_size = 0;
+  uint8_t * const buffer = new uint8_t[buffer_size];
+  bool error = false;
+
+  while( rest > 0 )
+    {
+    const int size = std::min( (long long)buffer_size, rest );
+    if( max_size >= 0 ) rest -= size;
+    const int rd = readblock( infd, buffer, size );
+    if( rd != size && errno )
+      { show_error( "Error reading input file", errno ); error = true; break; }
+    if( rd > 0 )
+      {
+      const int wr = writeblock( outfd, buffer, rd );
+      if( wr != rd )
+        { show_error( "Error writing output file", errno );
+          error = true; break; }
+      copied_size += rd;
+      }
+    if( rd < size ) break;				// EOF
+    }
+  delete[] buffer;
+  return ( !error && ( max_size < 0 || copied_size == max_size ) );
+  }
+
+
+/* Check archive type. If success, leave fd file pos at 0.
+   If remove_eof, remove EOF blocks, and leave fd file pos at EOF. */
+bool check_appendable( const int fd, const bool remove_eof )
   {
   struct stat st;
-  if( fstat( outfd, &st ) != 0 || !S_ISREG( st.st_mode ) ) return false;
-  uint8_t buf[header_size];
-  int rd = readblock( outfd, buf, header_size );
+  if( fstat( fd, &st ) != 0 || !S_ISREG( st.st_mode ) ) return false;
+  if( lseek( fd, 0, SEEK_SET ) != 0 ) return false;
+  enum { bufsize = header_size + ( header_size / 8 ) };
+  uint8_t buf[bufsize];
+  int rd = readblock( fd, buf, bufsize );
   if( rd == 0 && errno == 0 ) return true;	// append to empty archive
-  if( rd < min_member_size || ( rd != header_size && errno ) ) return false;
-  const Lzip_header * const p = (Lzip_header *)buf;	// shut up gcc
+  if( rd < min_member_size || ( rd != bufsize && errno ) ) return false;
+  const Lzip_header * const p = (const Lzip_header *)buf;	// shut up gcc
   if( !p->verify_magic() ) return false;
   LZ_Decoder * decoder = LZ_decompress_open();	// decompress first header
   if( !decoder || LZ_decompress_errno( decoder ) != LZ_ok ||
       LZ_decompress_write( decoder, buf, rd ) != rd ||
-      ( rd = LZ_decompress_read( decoder, buf, header_size ) ) <
-      magic_o + magic_l )
+      ( rd = LZ_decompress_read( decoder, buf, header_size ) ) != header_size )
     { LZ_decompress_close( decoder ); return false; }
   LZ_decompress_close( decoder );
   const bool maybe_eof = ( buf[0] == 0 );
   if( !verify_ustar_chksum( buf ) && !maybe_eof ) return false;
-  const long long end = lseek( outfd, 0, SEEK_END );
+  const long long end = lseek( fd, 0, SEEK_END );
   if( end < min_member_size ) return false;
 
   Lzip_trailer trailer;
-  if( seek_read( outfd, trailer.data, Lzip_trailer::size,
+  if( seek_read( fd, trailer.data, Lzip_trailer::size,
                  end - Lzip_trailer::size ) != Lzip_trailer::size )
     return false;
   const long long member_size = trailer.member_size();
@@ -94,9 +130,8 @@ bool check_appendable()
       ( maybe_eof && member_size != end ) ) return false;
 
   Lzip_header header;
-  if( seek_read( outfd, header.data, Lzip_header::size,
-                 end - member_size ) != Lzip_header::size )
-    return false;
+  if( seek_read( fd, header.data, Lzip_header::size,
+                 end - member_size ) != Lzip_header::size ) return false;
   if( !header.verify_magic() || !isvalid_ds( header.dictionary_size() ) )
     return false;
 
@@ -109,8 +144,11 @@ bool check_appendable()
   crc ^= 0xFFFFFFFFU;
   if( crc != data_crc ) return false;
 
-  if( lseek( outfd, end - member_size, SEEK_SET ) != end - member_size ||
-      ftruncate( outfd, end - member_size ) != 0 ) return false;
+  if( remove_eof )
+    { if( lseek( fd, end - member_size, SEEK_SET ) != end - member_size ||
+        ftruncate( fd, end - member_size ) != 0 ) return false; }
+  else
+    if( lseek( fd, 0, SEEK_SET ) != 0 ) return false;
   return true;
   }
 
@@ -290,10 +328,17 @@ int add_member( const char * const filename, const struct stat *,
                         S_IRWXU | S_IRWXG | S_IRWXO ) );
   const uid_t uid = ( cl_owner >= 0 ) ? (uid_t)cl_owner : st.st_uid;
   const gid_t gid = ( cl_group >= 0 ) ? (gid_t)cl_group : st.st_gid;
+  if( uid >= 2 << 20 || gid >= 2 << 20 )
+    { show_file_error( filename, "uid or gid is larger than 2_097_151." );
+      gretval = 1; return 0; }
   print_octal( header + uid_o, uid_l - 1, uid );
   print_octal( header + gid_o, gid_l - 1, gid );
+  const long long mtime = st.st_mtime;			// shut up gcc
+  if( mtime < 0 || mtime >= 1LL << 33 )
+    { show_file_error( filename, "mtime is out of ustar range [0, 8_589_934_591]." );
+      gretval = 1; return 0; }
+  print_octal( header + mtime_o, mtime_l - 1, mtime );
   unsigned long long file_size = 0;
-  print_octal( header + mtime_o, mtime_l - 1, st.st_mtime );
   Typeflag typeflag;
   if( S_ISREG( mode ) ) { typeflag = tf_regular; file_size = st.st_size; }
   else if( S_ISDIR( mode ) )
@@ -315,12 +360,15 @@ int add_member( const char * const filename, const struct stat *,
       delete[] buf;
       if( len != st.st_size )
       { show_file_error( filename, "Error reading link", (len < 0) ? errno : 0 );
-        gretval = 2; return 0; }
+        gretval = 1; return 0; }
       }
     }
   else if( S_ISCHR( mode ) || S_ISBLK( mode ) )
     {
     typeflag = S_ISCHR( mode ) ? tf_chardev : tf_blockdev;
+    if( major( st.st_dev ) >= 2 << 20 || minor( st.st_dev ) >= 2 << 20 )
+      { show_file_error( filename, "devmajor or devminor is larger than 2_097_151." );
+        gretval = 1; return 0; }
     print_octal( header + devmajor_o, devmajor_l - 1, major( st.st_dev ) );
     print_octal( header + devminor_o, devminor_l - 1, minor( st.st_dev ) );
     }
@@ -401,6 +449,43 @@ bool verify_ustar_chksum( const uint8_t * const buf )
     ustar_chksum( buf ) == strtoul( (const char *)buf + chksum_o, 0, 8 ) ); }
 
 
+int concatenate( const std::string & archive_name, const Arg_parser & parser,
+                 const int filenames )
+  {
+  if( !filenames )
+    { if( verbosity >= 1 ) show_error( "Nothing to concatenate." ); return 0; }
+  if( archive_name.empty() )
+    { show_error( "'--concatenate' is incompatible with '-f -'.", 0, true );
+      return 1; }
+  if( ( outfd = open_outstream( archive_name, false ) ) < 0 ) return 1;
+
+  int retval = 0;
+  for( int i = 0; i < parser.arguments(); ++i )		// copy archives
+    {
+    if( parser.code( i ) ) continue;			// skip options
+    const char * const filename = parser.argument( i ).c_str();
+    const int infd = open_instream( filename );
+    if( infd < 0 )
+      { show_file_error( filename, "Can't open input file", errno );
+        retval = 1; break; }
+    if( !check_appendable( infd, false ) )
+      { show_file_error( filename, "Not an appendable tar.lz archive." );
+        close( infd ); retval = 2; break; }
+    if( !check_appendable( outfd, true ) )
+      { show_error( "This does not look like an appendable tar.lz archive." );
+        close( infd ); retval = 2; break; }
+    if( !copy_file( infd, outfd ) || close( infd ) != 0 )
+      { show_file_error( filename, "Error copying archive", errno );
+        retval = 1; break; }
+    if( verbosity >= 1 ) std::fprintf( stderr, "%s\n", filename );
+    }
+
+  if( close( outfd ) != 0 && !retval )
+    { show_error( "Error closing archive", errno ); retval = 1; }
+  return retval;
+  }
+
+
 int encode( const std::string & archive_name, const Arg_parser & parser,
             const int filenames, const int level, const bool append )
   {
@@ -442,7 +527,7 @@ int encode( const std::string & archive_name, const Arg_parser & parser,
       { show_error( "'--append' is incompatible with '--uncompressed'.", 0, true );
         return 1; }
     if( ( outfd = open_outstream( archive_name, false ) ) < 0 ) return 1;
-    if( !check_appendable() )
+    if( !check_appendable( outfd, true ) )
       { show_error( "This does not look like an appendable tar.lz archive." );
         return 2; }
     }
@@ -462,7 +547,6 @@ int encode( const std::string & archive_name, const Arg_parser & parser,
     }
 
   int retval = 0;
-  std::string deslashed;		// arg without trailing slashes
   for( int i = 0; i < parser.arguments(); ++i )		// write members
     {
     const int code = parser.code( i );
@@ -472,6 +556,7 @@ int encode( const std::string & archive_name, const Arg_parser & parser,
       { show_file_error( filename, "Error changing working directory", errno );
         retval = 1; break; }
     if( code ) continue;				// skip options
+    std::string deslashed;		// arg without trailing slashes
     unsigned len = arg.size();
     while( len > 1 && arg[len-1] == '/' ) --len;
     if( len < arg.size() )
