@@ -530,7 +530,7 @@ error:
 
 
 int decode( const std::string & archive_name, const Arg_parser & parser,
-            const int filenames, const bool ignore_crc, const bool listing,
+            const int filenames, const bool listing, const bool missing_crc,
             const bool permissive )
   {
   const int infd = archive_name.size() ?
@@ -580,9 +580,8 @@ int decode( const std::string & archive_name, const Arg_parser & parser,
       if( !extended.parse( infd, header, permissive ) )
         { show_error( "Error in extended records. Skipping to next header." );
           extended.reset(); gretval = 2; }
-      else if( !extended.crc_present && !ignore_crc )
-        { show_error( "Missing CRC in extended records. Use --ignore-crc.",
-                      0, true ); return 2; }
+      else if( !extended.crc_present && missing_crc )
+        { show_error( "Missing CRC in extended records.", 0, true ); return 2; }
       prev_extended = true;
       continue;
       }
