@@ -61,7 +61,7 @@ const char * const program_name = "tarlz";
 const char * const program_year = "2018";
 const char * invocation_name = 0;
 
-enum Mode { m_none, m_append, m_create, m_extract, m_list };
+enum Mode { m_none, m_append, m_concatenate, m_create, m_extract, m_list };
 
 
 void show_help()
@@ -71,6 +71,7 @@ void show_help()
   std::printf( "\nOptions:\n"
                "  -h, --help                 display this help and exit\n"
                "  -V, --version              output version information and exit\n"
+               "  -A, --concatenate          append tar.lz archives to the end of an archive\n"
                "  -c, --create               create a new archive\n"
                "  -C, --directory=<dir>      change to directory <dir>\n"
                "  -f, --file=<archive>       use archive file <archive>\n"
@@ -85,7 +86,7 @@ void show_help()
                "      --solid                create solidly compressed archive\n"
                "      --group=<group>        use <group> name/id for added files\n"
                "      --owner=<owner>        use <owner> name/id for added files\n"
-               "      --ignore-crc           ignore missing CRC in extended records\n"
+               "      --missing-crc          exit with error status if missing extended CRC\n"
 //               "      --permissive           allow repeated extended headers and records\n"
                "      --uncompressed         don't compress the created archive\n"
                "\nExit status: 0 for a normal exit, 1 for environmental problems (file\n"
@@ -282,7 +283,7 @@ int main( const int argc, const char * const argv[] )
   std::string archive_name;
   Mode program_mode = m_none;
   int level = 6;		// compression level, < 0 = uncompressed
-  bool ignore_crc = false;
+  bool missing_crc = false;
   bool permissive = false;
   invocation_name = argv[0];
 
@@ -300,6 +301,7 @@ int main( const int argc, const char * const argv[] )
     { '7',  0,                Arg_parser::no  },
     { '8',  0,                Arg_parser::no  },
     { '9',  0,                Arg_parser::no  },
+    { 'A', "concatenate",     Arg_parser::no  },
     { 'c', "create",          Arg_parser::no  },
     { 'C', "directory",       Arg_parser::yes },
     { 'f', "file",            Arg_parser::yes },
@@ -314,7 +316,7 @@ int main( const int argc, const char * const argv[] )
     { opt_aso, "asolid",      Arg_parser::no  },
     { opt_dso, "dsolid",      Arg_parser::no  },
     { opt_grp, "group",       Arg_parser::yes },
-    { opt_crc, "ignore-crc",  Arg_parser::no  },
+    { opt_crc, "missing-crc", Arg_parser::no  },
     { opt_own, "owner",       Arg_parser::yes },
     { opt_per, "permissive",  Arg_parser::no  },
     { opt_sol, "solid",       Arg_parser::no  },
@@ -337,6 +339,7 @@ int main( const int argc, const char * const argv[] )
       case '0': case '1': case '2': case '3': case '4':
       case '5': case '6': case '7': case '8': case '9':
                 level = code - '0'; break;
+      case 'A': set_mode( program_mode, m_concatenate ); break;
       case 'c': set_mode( program_mode, m_create ); break;
       case 'C': break;					// skip chdir
       case 'f': if( sarg != "-" ) archive_name = sarg; break;
@@ -349,7 +352,7 @@ int main( const int argc, const char * const argv[] )
       case 'V': show_version(); return 0;
       case 'x': set_mode( program_mode, m_extract ); break;
       case opt_aso: cl_solid = 2; break;
-      case opt_crc: ignore_crc = true; break;
+      case opt_crc: missing_crc = true; break;
       case opt_dso: cl_solid = 1; break;
       case opt_grp: set_group( arg ); break;
       case opt_own: set_owner( arg ); break;
@@ -371,8 +374,9 @@ int main( const int argc, const char * const argv[] )
     case m_append:
     case m_create: return encode( archive_name, parser, filenames, level,
                                   program_mode == m_append );
+    case m_concatenate: return concatenate( archive_name, parser, filenames );
     case m_extract:
-    case m_list:   return decode( archive_name, parser, filenames, ignore_crc,
-                                  program_mode == m_list, permissive );
+    case m_list:   return decode( archive_name, parser, filenames,
+                            program_mode == m_list, missing_crc, permissive );
     }
   }
