@@ -111,12 +111,14 @@ extern int cl_solid;
 unsigned ustar_chksum( const uint8_t * const buf );
 bool verify_ustar_chksum( const uint8_t * const buf );
 class Arg_parser;
+int concatenate( const std::string & archive_name, const Arg_parser & parser,
+                 const int filenames );
 int encode( const std::string & archive_name, const Arg_parser & parser,
             const int filenames, const int level, const bool append );
 
 // defined in extract.cc
 int decode( const std::string & archive_name, const Arg_parser & parser,
-            const int filenames, const bool ignore_crc, const bool listing,
+            const int filenames, const bool listing, const bool missing_crc,
             const bool permissive );
 
 // defined in main.cc
