@@ -129,43 +129,59 @@ rm -f test.tar || framework_failure
 "${TARLZ}" --group=invalid_goup_name -tf ${test3_lz} 2> /dev/null
 [ $? = 1 ] || test_failed $LINENO
 
-"${TARLZ}" -tf ${eof_lz} || test_failed $LINENO
-"${TARLZ}" -xf ${eof_lz} || test_failed $LINENO
-"${TARLZ}" -tf ${in_tar_lz} > /dev/null || test_failed $LINENO
-"${TARLZ}" -xf ${in_tar_lz} || test_failed $LINENO
+"${TARLZ}" -tf ${eof_lz} --missing-crc || test_failed $LINENO
+"${TARLZ}" -xf ${eof_lz} --missing-crc || test_failed $LINENO
+"${TARLZ}" -tf ${in_tar_lz} --missing-crc > /dev/null || test_failed $LINENO
+"${TARLZ}" -xf ${in_tar_lz} --missing-crc || test_failed $LINENO
 cmp ${in} test.txt || test_failed $LINENO
 rm -f test.txt || framework_failure
 "${TARLZ}" -C nx_dir -tf ${in_tar} > /dev/null || test_failed $LINENO
-"${TARLZ}" -xf ${in_tar} || test_failed $LINENO
+"${TARLZ}" -xf ${in_tar} --missing-crc || test_failed $LINENO
 cmp ${in} test.txt || test_failed $LINENO
 rm -f test.txt || framework_failure
 
 rm -f foo bar baz || framework_failure
-"${TARLZ}" -xf ${test3_lz} || test_failed $LINENO
+"${TARLZ}" -xf ${test3_lz} --missing-crc || test_failed $LINENO
 [ -e foo ] || test_failed $LINENO
 [ -e bar ] || test_failed $LINENO
 [ -e baz ] || test_failed $LINENO
 rm -f foo bar baz || framework_failure
-"${TARLZ}" -xf ${test3} || test_failed $LINENO
+"${TARLZ}" -xf ${test3} --missing-crc || test_failed $LINENO
 [ -e foo ] || test_failed $LINENO
 [ -e bar ] || test_failed $LINENO
 [ -e baz ] || test_failed $LINENO
 rm -f foo bar baz || framework_failure
-"${TARLZ}" -qxf ${test3dir_lz} || test_failed $LINENO
+"${TARLZ}" -qxf ${test3dir_lz} --missing-crc || test_failed $LINENO
 [ -e dir/foo ] || test_failed $LINENO
 [ -e dir/bar ] || test_failed $LINENO
 [ -e dir/baz ] || test_failed $LINENO
 rm -rf dir || framework_failure
 
+cat ${in_tar_lz} > out.tar.lz || framework_failure
+"${TARLZ}" -Af out.tar.lz ${test3_lz} || test_failed $LINENO
+"${TARLZ}" -xf out.tar.lz || test_failed $LINENO
+[ -e test.txt ] || test_failed $LINENO
+[ -e foo ] || test_failed $LINENO
+[ -e bar ] || test_failed $LINENO
+[ -e baz ] || test_failed $LINENO
+touch aout.tar.lz || framework_failure		# concatenate to empty file
+"${TARLZ}" -Af aout.tar.lz ${in_tar_lz} ${test3_lz} || test_failed $LINENO
+cmp out.tar.lz aout.tar.lz || test_failed $LINENO
+cat ${in_tar_lz} > aout.tar.lz || framework_failure
+"${TARLZ}" -Aqf aout.tar.lz ${test3_lz} ${test3}
+[ $? = 2 ] || test_failed $LINENO
+cmp out.tar.lz aout.tar.lz || test_failed $LINENO
+rm -f test.txt foo bar baz out.tar.lz aout.tar.lz || framework_failure
+
 cat ${in} > test.txt || framework_failure
 "${TARLZ}" -0 -cf out.tar.lz test.txt || test_failed $LINENO
 rm -f test.txt || framework_failure
-"${TARLZ}" -xf out.tar.lz || test_failed $LINENO
+"${TARLZ}" -xf out.tar.lz --missing-crc || test_failed $LINENO
 cmp ${in} test.txt || test_failed $LINENO
 cat ${in} > test.txt || framework_failure
 "${TARLZ}" --uncompressed -cf out.tar test.txt || test_failed $LINENO
 rm -f test.txt || framework_failure
-"${TARLZ}" -xf out.tar || test_failed $LINENO
+"${TARLZ}" -xf out.tar --missing-crc || test_failed $LINENO
 cmp ${in} test.txt || test_failed $LINENO
 rm -f test.txt out.tar out.tar.lz || framework_failure
 
@@ -175,7 +191,7 @@ printf "baz" > baz || framework_failure
 "${TARLZ}" -qcf out.tar.lz foo bar baz
 [ $? = 1 ] || test_failed $LINENO
 rm -f foo bar baz || framework_failure
-"${TARLZ}" -xf out.tar.lz || test_failed $LINENO
+"${TARLZ}" -xf out.tar.lz --missing-crc || test_failed $LINENO
 [ -e foo ] || test_failed $LINENO
 [ ! -e bar ] || test_failed $LINENO
 [ -e baz ] || test_failed $LINENO
